#! /usr/bin/python
# Copyright 2014 Canonical
# All Rights Reserved


"""The New York Times scope."""

from ..scopes import SimpleApp
from urllib2 import urlopen
from collections import Counter

import json
import urllib

SEARCH_API_KEY = '37ccc06a0431422665956a67b235bb91:1:68718960'
WIRE_API_KEY = 'e2051c7ce26742ae617baea8ba60221e:16:68718960'
POPULAR_API_KEY = '3ee052d66db895b949e6abe3ec80e4eb:6:68718960'

# Standard category renderer
RENDERER_DEFAULT = {
    'template': {'renderer': 'grid', 'overlay-mode': None},
    'components': {'title':'title','art':'art'},
}

def _search_method(hint):
    # Pick a NYTimes API to use
    if hint == "popular":
        key = POPULAR_API_KEY
        sections = "all-sections"
        uri = "http://api.nytimes.com/svc/mostpopular/v2/mostviewed/%s/7.json?api-key=%s" % (sections, key,)
    elif hint == "wire":
        key = WIRE_API_KEY
        sections = "all"
        uri = "http://api.nytimes.com/svc/news/v3/content/all/%s.json?api-key=%s" % (sections, key,)
    else:
        key = SEARCH_API_KEY
        sec = ""
        uri = 'http://api.nytimes.com/svc/search/v2/articlesearch.json?sort=newest&api-key=%s%s' % (key, sec,)
    return uri

class App(SimpleApp):
    """The NYTimes app."""
    
    # Declare categories
    cat_ids = ['latest', 'popular']
    categories = {
        cat_ids[0]: {
            'id': cat_ids[0],
            'title': 'Latest News',
            'render_template': json.dumps(RENDERER_DEFAULT),
            'icon': "FIXME",
        },
        cat_ids[1]: {
            'id': cat_ids[1],
            'title': 'Popular Stories',
            'render_template': json.dumps(RENDERER_DEFAULT),
            'icon': "FIXME",
        },
    }

    def search(self, **kwargs):
        calls = []
        search = kwargs['query']
        limit = kwargs['limit']
        
        # Pick a search method depending on the query
        if not search or len(search) < 3:
            calls.append(_search_method("wire"))
            calls.append(_search_method("popular"))
        else:
            q = urllib.quote(search.encode("utf8"))
            uri = "%s&q=%s" % (_search_method("search"), q,)
            calls.append(uri)
        
        # Send our categories
        for c in self.cat_ids:
            yield dict(category=self.categories[c])

        # Iterate through the various queries we need to do
        for uri in calls:
            print (uri)
            try:
                response = urlopen(uri).read()
                data = json.loads(response.decode('utf-8'))
            except Exception as error:
                print (error)
            keywords_list = []
            try:
                items = data['response']['docs']
            except:
                items = data['results']
            for d in items[:limit]:

                # Look for tags associated to articles
                try:
                    for k in d['keywords']:
                        if k['value'] != "" and k['value'].lower() != search.lower():
                            keywords_list.append(k['value'])
                except:
                    pass
                icon = "http://thesportsgene.com/wp-content/uploads/2013/08/NY-Times-logo-150x150.jpg"

                # Try several ways of getting an image in order of preference
                try:
                    for m in d['multimedia']:
                        if m['format'] == 'thumbLarge':
                            icon = m['url']
                        elif m['format'] == 'Normal':
                            icon = m['url']
                        elif m['format'] == 'Large':
                            icon = m['url']
                        elif m['format'] == 'square320':
                            icon = m['url']
                        elif m['format'] == 'mediumThreeByTwo210':
                            icon = m['url']
                        elif m['format'] == 'Standard Thumbnail':
                            icon = m['url']
                        elif m['format'] == 'square640':
                            icon = m['url']
                        break
                except:
                    try:
                        if len(d['media']) > 0:
                            if d['media'][0]['type'] == "image":
                                for m in d['media'][0]['media-metadata']:
                                    if m['format'] == 'thumbLarge':
                                        icon = m['url']
                                    elif m['format'] == 'Normal':
                                        icon = m['url']
                                    elif m['format'] == 'Large':
                                        icon = m['url']
                                    elif m['format'] == 'square320':
                                        icon = m['url']
                                    elif m['format'] == 'mediumThreeByTwo210':
                                        icon = m['url']
                                    elif m['format'] == 'Standard Thumbnail':
                                        icon = m['url']
                                    elif m['format'] == 'square640':
                                        icon = m['url']
                                    break
                    except:
                        for m in d['multimedia']:
                            if m['subtype'] == 'thumbnail':
                                icon = m['url']
                            elif m['subtype'] == 'wide':
                                icon = m['url']
                            elif m['subtype'] == 'xlarge':
                                icon = m['url']
                            break
                        icon = "http://www.nytimes.com/" + icon

                # Article search result or newswire
                try:
                    url = d['web_url']
                    title = d['headline']['main']
                    comment = d['snippet']
                except:
                    url = d['url']
                    title = d['title']
                    comment = d['abstract']

                # Create result
                yield dict(result={
                    'cat_id': self.cat_ids[calls.index(uri)],
                    'uri': url,
                    'dnd_uri': url,
                    'title': title,
                    'art': icon,
                    'comment':comment,
                })

            # List most used tags for annotations
            topics = Counter(keywords_list).most_common(3)
            if len(topics) > 0:
                for t in topics:
                    yield dict(result={
                    'cat_id': self.cat_ids[0],
                    'uri': t[0],
                    'dnd_uri': t[0],
                    'title': t[0] + " (link annotation)",
                    'art': "",
                    'comment':'#FIXME This will be a link annotation under the Latest News category',
                    })


    def preview(self, **kwargs):
        """Do the preview."""
        # FIXME: do this
        print "======= real preview", kwargs
