# Copyright 2014 Canonical
# All Rights Reserved

"""The Wikipedia scope."""

import re
import urllib

from urllib2 import urlopen, Request
from xml.dom.minidom import parseString

from ..scopes import SimpleApp, CATEGORY_DEFAULT
from ..utils import full_icon_url


# We use the opensearch action as that seems to be the only action that
# searches only on page names -- the standard search function performs a
# full-text search and may return less relevant pages first.
SEARCH_URL = (
    'http://%(language)s.wikipedia.org'
    '/w/api.php?action=opensearch&limit=%(limit)s&format=xml&search=%(query)s'
)

# these are the default image sizes for the search and preview
IMAGE_DISPLAY_SIZE = '96'

# the regex to replace the image size in the returned URL
RE_IMAGE_SIZE = re.compile(r'(?<=\/)\d+(?=px-)')

HEADERS = {
    'User-Agent': "Ubuntu One Wikipedia Scope (u1di@canonical.com)",
    'Accept': '*/*',
}

def _get_search(query, limit, language):
    query = urllib.quote(query.encode("utf8"))
    url = SEARCH_URL % dict(language=language, limit=limit, query=query)
    req = Request(url, headers=HEADERS)
    return urlopen(req).read()


class App(SimpleApp):
    """The Wikipedia app."""

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query']
        locale = kwargs['locale']
        limit = kwargs['limit']

        # get the language from the locale
        if locale is None:
            language = 'en'
        else:
            language = locale.split('_', 1)[0]

        # do the search, and get the result as an xml
        raw = _get_search(query, limit, language)
        xml = parseString(raw)

        # send the category we'll use
        yield dict(category=self.categories[CATEGORY_DEFAULT])

        for item in xml.getElementsByTagName('Item'):
            img = item.getElementsByTagName('Image')
            if img:
                # the image for the search result
                img_source = img[0].getAttribute("source")
                art = RE_IMAGE_SIZE.sub(IMAGE_DISPLAY_SIZE, img_source)
            else:
                art = full_icon_url('service-wikipedia.svg')

            title = item.getElementsByTagName('Text')[0].firstChild.data
            url = item.getElementsByTagName('Url')[0].firstChild.data
            yield dict(result={
                'cat_id': CATEGORY_DEFAULT,
                'uri': url,
                'dnd_uri': url,
                'title': title,
                'art': art,
            })

    def preview(self, **kwargs):
        """Do the preview."""
        # FIXME: do this
        print "======= real preview", kwargs
